from ..base import Base
from collections import namedtuple

Means = namedtuple('Means', 'type')

class Car(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "index":
        self.index = int(value)
      case "number":
        self.number = value
      case "means":
        self.means = Means(
          type = value["Type"])
      case _:
        raise ValueError(f"key: {key} is not defined in Car")



from ekispert.models.point import Point
from ..base import Base

class TeikiRouteSection(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "repaymentticketindex":
        if value is not None:
          self.repayment_ticket_index = int(value)
      case "point":
        points = self.get_as_array(value)
        if points is not None and len(points) > 0:
          self.points = []
          for point in points:
            self.points.append(Point(point))
      case _:
        raise ValueError(f"key: {key} is not defined in TeikiRouteSection")



from datetime import datetime
from ekispert.models.cource import Course
from ..base import Base
from typing import List, Literal

Gcs = Literal["tokyo", "wgs84"]

class CoursePlainQuery(Base):
  base_path: str = '/v1/json/search/course/plain'

  def __init__(self, client):
    super().__init__()
    self.client = client
    self.from_ : str | int = None
    self.to : str | int = None
    self.via : str | int = None
    self.date : datetime = datetime.today()
    self.plane : bool = True
    self.shinkansen : bool = True
    self.limited_express : bool = True
    self.bus : bool = True
    self.gcs : Gcs = 'tokyo'

  def execute(self) -> List[Course]:
    data = self.client.get(self.base_path, self.generate_params())
    result_set = data['ResultSet']
    courses = self.get_as_array(result_set['Course'])
    if len(courses) == 0:
      return []
    results = []
    for course in courses:
      results.append(Course(course))
    return results

  def generate_params(self) -> dict:
    params = {
      'key': self.client.api_key,
    }
    if self.from_ is None:
      raise ValueError("from is required")
    params['from'] = self.from_
    
    if self.to is None:
      raise ValueError("to is required")
    params['to'] = self.to
    if self.via:
      params['via'] = self.via
    if self.date:
      params['date'] = self.date.strftime('%Y%m%d')
    if self.gcs:
      params['gcs'] = self.gcs
    params['plane'] = self.get_as_boolean_string(self.plane)
    params['shinkansen'] = self.get_as_boolean_string(self.shinkansen)
    params['limitedExpress'] = self.get_as_boolean_string(self.limited_express)
    params['bus'] = self.get_as_boolean_string(self.bus)
    params['gcs'] = self.gcs
    return params


from ekispert.models.change import Change
from ekispert.models.operational_state import OperationalState
from ekispert.models.stop import Stop
from ..base import Base

from collections import namedtuple

Comment = namedtuple('Comment', 'type')
LineSymbol = namedtuple('LineSymbol', 'code name')
Type = namedtuple('Type', 'text detail')
class Line(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "chargeindex":
        self.charge_index = value
      case "direction":
        self.direction = value
      case "distance":
        self.distance = int(value)
      case "exhaustco2":
        self.exhaust_co2 = int(value)
      case "exhaustco2atpassengercar":
        self.exhaust_co2_at_passenger_car = int(value)
      case "fareindex":
        self.fare_index = value
      case "stopstationcount":
        self.stop_station_count = int(value)
      case "teiki1index":
        self.teiki1_index = value
      case "teiki3index":
        self.teiki3_index = value
      case "teiki6index":
        self.teiki6_index = value
      case "teiki12index":
        self.teiki12_index = value
      case "timeonboard":
        self.time_on_board = int(value)
      case "track":
        self.track = value
      case "cars":
        self.cars = int(value)
      case "arrivalstate":
        self.arrival_state = OperationalState(value)
      case "departurestate":
        self.departure_state = OperationalState(value)
      case "change":
        self.change = Change(value)
      case "color":
        self.color = value
      case "comment":
        self.comment = Comment(
          type = value["type"])
      case "destination":
        self.destination = value
      case "linesymbol":
        self.line_symbol = LineSymbol(
          code = value["code"],
          name = value["Name"])
      case "name":
        self.name = value
      case "number":
        self.number = value
      case "timereliability":
        self.time_reliability = value
      case "type":
        if isinstance(value, str):
          self.type = Type(
            text = value,
            detail = None)
        else:
          self.type = Type(
            text = value["text"],
            detail = value["detail"])
      case "typicalname":
        self.typical_name = value
      case "stop":
        self.stop = Stop(value)
      case "trainid":
        self.train_id = value
      case _:
        raise ValueError(f"key: {key} is not defined in Line")




from datetime import datetime

from ekispert.models.repayment_ticket import RepaymentTicket
from ..base import Base

class RepaymentList(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "repaymentdate":
        # str to datetime
        if value is not None:
          self.repayment_date = datetime.strptime(value, '%Y-%m-%d')
      case "validityperiod":
        # str to int
        if value is not None:
          self.validity_period = int(value)
      case "startdate":
        # str to datetime
        if value is not None:
          self.start_date = datetime.strptime(value, '%Y-%m-%d')
      case "buydate":
        # str to datetime
        if value is not None:
          self.buy_date = datetime.strptime(value, '%Y-%m-%d')
      case "repaymentticket":
        repayment_tickets = self.get_as_array(value)
        if repayment_tickets is not None and len(repayment_tickets) > 0:
          self.repayment_tickets = []
          for repayment_ticket in repayment_tickets:
            self.repayment_tickets.append(RepaymentTicket(repayment_ticket))
      case _:
        raise ValueError(f"key: {key} is not defined in RepaymentList")



from ..base import Base

class TripOption(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "fullremark":
        self.full_remark = value
      case "remark":
        self.remark = value
      case "expectedfullremark":
        self.expected_full_remark = value
      case "expectedremark":
        self.expected_remark = value
      case "includedincharge":
        self.included_in_charge = True
      case "rate":
        self.rate = value
      case _:
        raise ValueError(f"key: {key} is not defined in Cost")


from ekispert.models.pass_status import PassStatus
from ekispert.models.price import Price
from ekispert.models.relation import Relation
from ekispert.models.route import Route
from ekispert.models.teiki import Teiki
from ..base import Base

class Course(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "searchtype":
        self.search_type = value
      case "datatype":
        self.data_type = value
      case "serializedata":
        self.serialize_data = value
      case "passstatus":
        # check value is array or not
        value = self.get_as_array(value)
        self.pass_statuses = []
        for v in value:
          if v is not None:
            self.pass_statuses.append(PassStatus(v))
      case "price":
        value = self.get_as_array(value)
        self.prices = []
        for v in value:
          if v is not None:
            self.prices.append(Price(v))
      case "relation":
        # value is array or not
        value = self.get_as_array(value)
        self.relations = []
        for v in value:
          if v is not None:
            self.relations.append(Relation(value))
      case "teiki":
        self.teiki = Teiki(value)
      case "route":
        value = self.get_as_array(value)
        self.routes = []
        for v in value:
          if v is not None:
            self.routes.append(Route(v))
      case _:
        raise ValueError(f"key: {key} is not defined in Course")


from ekispert.models.trip_option import TripOption
from ..base import Base

from collections import namedtuple

Rate = namedtuple('Rate', 'text area')

class Price(Base):
  def __init__(self, data = None):
    super().__init__()
    if data is None:
      return
    self.sets(data)

  def sets(self, data: dict):
    for key in data:
      self.set(key, data[key])

  def set(self, key: str, value: any):
    match key.lower():
      case "farerevisionstatus":
        self.fare_revision_status = value
      case "fromlineindex":
        self.from_line_index = int(value)
      case "index":
        self.index = int(value)
      case "kind":
        self.kind = value
      case "nikukanteikiindex":
        self.nikukanteiki_index = int(value)
      case "offpeakteiki":
        self.offpeak_teiki = value
      case "passclassindex":
        self.pass_class_index = int(value)
      case "relationindex":
        self.relation_index = int(value)
      case "selected":
        if value == "true":
          self.selected = True
        else:
          self.selected = False
      case "tolineindex":
        self.to_line_index = int(value)
      case "vehicleindex":
        self.vehicle_index = int(value)
      case "name":
        self.name = value
      case "oneway":
        # if value is string, it is not array
        if isinstance(value, str):
          self.one_way = int(value)
        else:
          self.one_way = TripOption(value)
      case "rate":
        self.rate = Rate(
          text = value["text"],
          area = value["area"]
        )
      case "revisionstatus":
        self.revision_status = value
      case "revisionstatuscomment":
        self.revision_status_comment = value
      case "round":
        if isinstance(value, str):
          self.round = int(value)
        else:
          self.round = TripOption(value)
      case "type":
        self.type = value
      case _:
        raise ValueError(f"key: {key} is not defined in Price")


from datetime import datetime
from ekispert.models.cource import Course
from ..base import Base
from typing import List, Literal

Gcs = Literal["tokyo", "wgs84"]
SearchType = Literal["departure", "arrival", "lastTrain", "firstTrain", "plain"]
Sort = Literal["ekispert", "price", "time", "teiki", "transfer", "co2", "teiki1", "teiki3", "teiki6"]
OffpeakTeikiMode = Literal["offpeakTime", "peakTime"]

class CourseExtremeQuery(Base):
  base_path: str = '/v1/json/search/course/extreme'

  def __init__(self, client):
    super().__init__()
    self.client = client
    self.via_list : List[str] = []
    self.exclude_same_line_station : bool = False
    self.fixed_rail_list : List[str] = []
    self.fixed_rail_direction_list : List[str] = []
    self.date : datetime = datetime.today()
    self.time : datetime = datetime.now()
    self.search_type : SearchType = 'departure'
    self.sort : Sort = 'ekispert'
    self.answer_count : int = 5
    self.search_count : int = 5
    self.condition_detail : List[str] = ['T32212332323191', 'F332112212000010', 'A23121141']
    self.corporation_bind : List[str] = []
    self.interrupt_corporation_list : List[str] = []
    self.interrupt_rail_list : List[str] = []
    self.interrupt_operation_line_code_list : List[str] = []
    self.interrupt_transfer_station_code_list : List[str] = []
    self.result_detail : str = None
    self.add_operation_line_pattern : bool = False
    self.check_engine_version : bool = True
    self.assign_teiki_serialize_data : str = None
    self.assign_route : List[str] = []
    self.assign_detail_route : List[str] = []
    self.offpeak_teiki_mode : OffpeakTeikiMode = None
    self.assign_pass_class_index : int = None
    self.coupon : str = None
    self.bring_assignment_error : bool = None
    self.add_assign_status : bool = None
    self.add_change : bool = None
    self.add_stop : bool = None
    self.add_seat_type : bool = None
    self.gcs : Gcs = 'tokyo'

  def execute(self) -> List[Course]:
    data = self.client.get(self.base_path, self.generate_params())
    result_set = data['ResultSet']
    courses = self.get_as_array(result_set['Course'])
    if len(courses) == 0:
      return []
    results = []
    for course in courses:
      results.append(Course(course))
    return results

  def generate_params(self) -> dict:
    params = {
      'key': self.client.api_key,
    }
    if len(self.via_list) > 0:
      params['viaList'] = ':'.join(self.via_list)
    if self.exclude_same_line_station is not None:
      params['excludeSameLineStation'] = self.get_as_boolean_string(self.exclude_same_line_station)
    if len(self.fixed_rail_list) > 0:
      params['fixedRailList'] = ':'.join(self.fixed_rail_list)
    if len(self.fixed_rail_direction_list) > 0:
      params['fixedRailDirectionList'] = ':'.join(self.fixed_rail_direction_list)
    if self.date is not None: 
      params['date'] = self.date.strftime('%Y%m%d')
    if self.time is not None:
      params['time'] = self.time.strftime('%H%M')
    if self.search_type is not None:
      params['searchType'] = self.search_type
    if self.sort is not None:
      params['sort'] = self.sort
    if self.answer_count is not None:
      params['answerCount'] = self.answer_count
    if self.search_count is not None:
      params['searchCount'] = self.search_count
    if len(self.condition_detail) > 0:
      params['conditionDetail'] = ':'.join(self.condition_detail)
    if len(self.corporation_bind) > 0:
      params['corporationBind'] = ':'.join(self.corporation_bind)
    if len(self.interrupt_corporation_list) > 0:
      params['interruptCorporationList'] = ':'.join(self.interrupt_corporation_list)
    if len(self.interrupt_rail_list) > 0:
      params['interruptRailList'] = ':'.join(self.interrupt_rail_list)
    if len(self.interrupt_operation_line_code_list) > 0:
      params['interruptOperationLineCodeList'] = ':'.join(self.interrupt_operation_line_code_list)
    if len(self.interrupt_transfer_station_code_list) > 0:
      params['interruptTransferStationCodeList'] = ':'.join(self.interrupt_transfer_station_code_list)
    if self.result_detail is not None:
      params['resultDetail'] = self.result_detail
    if self.add_operation_line_pattern is not None:
      params['addOperationLinePattern'] = self.get_as_boolean_string(self.add_operation_line_pattern)
    if self.check_engine_version is not None:
      params['checkEngineVersion'] = self.get_as_boolean_string(self.check_engine_version)
    if self.assign_teiki_serialize_data is not None:
      params['assignTeikiSerializeData'] = self.assign_teiki_serialize_data
    if len(self.assign_route) > 0:
      params['assignRoute'] = ':'.join(self.assign_route)
    if len(self.assign_detail_route) > 0:
      params['assignDetailRoute'] = ':'.join(self.assign_detail_route)
    if self.offpeak_teiki_mode is not None:
      params['offpeakTeikiMode'] = self.offpeak_teiki_mode
    if self.assign_pass_class_index is not None:
      params['assignPassClassIndex'] = self.assign_pass_class_index
    if self.coupon is not None:
      params['coupon'] = self.coupon
    if self.bring_assignment_error is not None:
      params['bringAssignmentError'] = self.get_as_boolean_string(self.bring_assignment_error)
    if self.add_assign_status is not None:
      params['addAssignStatus'] = self.get_as_boolean_string(self.add_assign_status)
    if self.add_change is not None:
      params['addChange'] = self.get_as_boolean_string(self.add_change)
    if self.add_stop is not None:
      params['addStop'] = self.get_as_boolean_string(self.add_stop)
    if self.add_seat_type is not None:
      params['addSeatType'] = self.get_as_boolean_string(self.add_seat_type)
    if self.gcs is not None:
      params['gcs'] = self.gcs
    return params


class Base:
  def get_as_boolean_string(self, data: bool) -> str:
    if data:
      return 'true'
    else:
      return 'false'

  def get_as_array(self, data) -> list:
    if isinstance(data, list):
      return data
    else:
      return [data]
